#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2013, a
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# * Redistributions of source code must retain the above copyright notice, 
#   this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice, 
#   this list of conditions and the following disclaimer in the documentation 
#   and/or other materials provided with the distribution.
# * Neither the name of the <organization> nor the　names of its contributors 
#   may be used to endorse or promote products derived from this software 
#   without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import sys
sys.path.append('.')

from MQEasy00 import EFace
import copy

cEnd = 0

def takaku2tri(vi, o, tf):
  f = EFace()
  f.load(tf, o)
  f.rotateSpecial(vi, 1)
  tri = copy.deepcopy(f)
  tri.delVertex(3, len(tri.index)-3)
  tri.writeNew(MQSystem)
  f.delVertex(1, 1)
  f.replaceNew()
  return tri.f

srcVI = []
dstVI = []


def getHalfVI(vi):
  global srcVI, dstVI
  return dstVI[srcVI.index(vi)]

def triCutter(vi, tf, o):
  global srcVI, dstVI
  tri = EFace()
  tri.load(tf, o)
  tri.rotateSpecial(vi, 1)
  if tri.index[0] in srcVI:
    tri.splitEdge(0, 0.5, MQSystem, getHalfVI(tri.index[0]))
  else:
    tri.splitEdge(0, 0.5, MQSystem)
    srcVI.append(tri.index[0])
    dstVI.append(tri.index[1])
  if tri.index[3] in srcVI:
    tri.splitEdge(2, 0.5, MQSystem, getHalfVI(tri.index[3]))
  else:
    tri.splitEdge(2, 0.5, MQSystem)
    srcVI.append(tri.index[4])
    dstVI.append(tri.index[3])
  rect = copy.deepcopy(tri)
  rect.delVertex(2)
  tri.delVertex(0)
  tri.delVertex(3)
  tri.writeNew(MQSystem)
  rect.replaceNew()
  return tri.f

def setudan(v, o):
  global cEnd
  if v.ref==0:
    return
  vi = o.getVertexIndexFromUniqueID(v.id)
  
  for tfi in v.faces:
    tri = o.face[tfi]
    if tri.numVertex > 3:
      tri = takaku2tri(vi, o, tri)
  
  for tfi in v.faces:
    tri = o.face[tfi]
    if tri.numVertex==3:
      triCutter(vi, tri, o)
  
  v.select = 0
  cEnd+=1

doc = MQSystem.getDocument()

for o in doc.object:
  for v in o.vertex:
    if v.select == 1:
      setudan(v, o)

MQSystem.println("処理頂点数: "+str(cEnd))