#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2013, a
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# * Redistributions of source code must retain the above copyright notice, 
#   this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice, 
#   this list of conditions and the following disclaimer in the documentation 
#   and/or other materials provided with the distribution.
# * Neither the name of the <organization> nor the　names of its contributors 
#   may be used to endorse or promote products derived from this software 
#   without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import copy
import MQSystem

def copy2List(srcList):
  retList = []
  for v in srcList:
    retList.append(v)
  return retList

def rotateListL(lst, i):
  return lst[i:] + lst[:i]

def setAllCoord(f, list):
  c=0
  for v in list:
    f.setCoord(c, v)
    c+=1

def setAllColor(f, list):
  c=0
  for v in list:
    f.setColor(c, v)
    c+=1

def setAllAlpha(f, list):
  c=0
  for v in list:
    f.setAlpha(c, v)
    c+=1

def setAllCrease(f, list):
  c=0
  for v in list:
    f.setEdgeCrease(c, v)
    c+=1

class EFace:
  def __deepcopy__(self, memo):
    newF = EFace()
    newF.f = self.f
    newF.obj = self.obj
    newF.index = self.index[:]
    newF.coord = self.coord[:]
    newF.color = self.color[:]
    newF.alpha = self.alpha[:]
    newF.crease = self.crease[:]
    newF.material = copy.deepcopy(self.material, memo)
    newF.select = copy.deepcopy(self.select, memo)
    return newF
  
  def load(self, f, obj):
    self.f = f
    self.obj = obj
    self.index = copy2List(f.index)
    self.coord = copy2List(f.coord)
    self.color = copy2List(f.color)
    self.alpha = copy2List(f.alpha)
    self.crease = copy2List(f.crease)
    self.material = f.material
    self.select = f.select
  
  # addVertexやdelVertexしている場合は失敗します。代わりにwriteNew、replaceNewを使ってください
  def writeOverWithoutIndex(self):
    setAllCoord(self.f, self.coord)
    setAllColor(self.f, self.color)
    setAllAlpha(self.f, self.alpha)
    setAllCrease(self.f, self.crease)
    self.f.material = self.material
    self.f.select = self.select
  
  def writeNew(self, mqsys):
    fI = self.obj.addFace(self.index)
    self.f = self.obj.face[fI]
    self.writeOverWithoutIndex()
  
  def replaceNew(self):
    delFaceId = self.f.id
    self.f = self.obj.face[self.obj.addFace(self.index)]
    self.writeOverWithoutIndex()
    self.obj.deleteFace(self.obj.getFaceIndexFromUniqueID(delFaceId))
  
  def addVertex(self, n, index, coord, color, alpha, crease):
    self.index.insert(n, index)
    self.coord.insert(n, coord)
    self.color.insert(n, color)
    self.alpha.insert(n, alpha)
    self.crease.insert(n, crease)
  
  def delVertex(self, n, size=1):
    if size<=0:
      return
    del self.index[n:n+size]
    del self.coord[n:n+size]
    del self.color[n:n+size]
    del self.alpha[n:n+size]
    del self.crease[n:n+size]
  
  def rotateL(self, n):
    if n==0:
      return
    self.index = rotateListL(self.index, n)
    self.coord = rotateListL(self.coord, n)
    self.color = rotateListL(self.color, n)
    self.alpha = rotateListL(self.alpha, n)
    self.crease = rotateListL(self.crease, n)
  
  def rotateR(self, n):
    if n==0:
      return
    self.rotateL(len(self.index)-n)
  
  def rotateSpecial(self, vi, newIdx):
    oldIdx = self.index.index(vi)
    if oldIdx < newIdx:
      self.rotateR(newIdx-oldIdx)
    else:
      self.rotateL(oldIdx-newIdx)
  
  def splitEdge(self, idx1, ratio1, mqsys, newIndex=None):
    idx2 = idx1+1
    if idx2 == len(self.index):
      idx2 = 0
    if newIndex==None:
      newIndex = self.ratioPoint(self.obj.vertex[self.index[idx1]].getPos(), self.obj.vertex[self.index[idx2]].getPos(), ratio1, self.obj, mqsys)
    newCoord = self.ratioMQCoordinate(self.coord[idx1], self.coord[idx2], ratio1, mqsys)
    newRatio = self.ratioMQColor(self.color[idx1], self.color[idx2], ratio1, mqsys)
    ratio2 = 1.0-ratio1
    newAlpha = min(self.alpha[idx1]*ratio1+self.alpha[idx2]*ratio2, 1.0)
    self.addVertex(idx2, newIndex, newCoord, newRatio, newAlpha, self.crease[0])
  
  def ratioPoint(self, pos1, pos2, ratio1, o, mqsys):
    mqsys.println("ratioPoint")
    ratio2 = 1.0-ratio1
    return o.addVertex(mqsys.newPoint(pos1.x*ratio1+pos2.x*ratio2, pos1.y*ratio1+pos2.y*ratio2, pos1.z*ratio1+pos2.z*ratio2))
  
  def ratioMQCoordinate(self, coord1, coord2, ratio1, mqsys):
    ratio2 = 1.0-ratio1
    return mqsys.newCoordinate(min(coord1.u*ratio1+coord2.u*ratio2, 1.0), min(coord1.v*ratio1+coord2.v*ratio2, 1.0))
  
  def ratioMQColor(self, color1, color2, ratio1, mqsys):
    ratio2 = 1.0-ratio1
    return mqsys.newColor(min(color1.red*ratio1+color2.red*ratio2, 1.0), min(color1.green*ratio1+color2.green*ratio2, 1.0), min(color1.blue*ratio1+color2.blue*ratio2, 1.0))
